angular
  .module('gu.service', [])
  .factory('AuthService', function($q, $http, $localStorage, API_ENDPOINT) {
    function urlBase64Decode(str) {
      var output = str.replace('-', '+').replace('_', '/');
      switch (output.length % 4) {
        case 0:
          break;
        case 2:
          output += '==';
          break;
        case 3:
          output += '=';
          break;
        default:
          throw 'Illegal base64url string!';
      }
      return window.atob(output);
    }

    function getClaimsFromToken() {
      var token = $localStorage.token;
      var user = {};
      if (typeof token !== 'undefined') {
        var encoded = token.split('.')[1];
        user = JSON.parse(urlBase64Decode(encoded));
      }
      return user;
    }

    function clearCache() {
      tokenClaims = {};
      delete $localStorage.token;
      delete $localStorage.heartNews;
      delete $localStorage.likedNews;
      delete $localStorage.focusedJournals;
    }

    return {
        testLogin: function(data, success, error) {
            $http
                .post(API_ENDPOINT.TEST_LOGIN, data)
                .then(function(response) {
                    $localStorage.token = response.data.token;
                    $localStorage.heartNews = response.data.heartNews;
                    $localStorage.likedNews = response.data.likedNews;
                    $localStorage.focusedJournals = response.data.focusedJournals;
                    success(response);
                })
                .catch(error);
        },
      login: function(data, success, error) {
        $http
        .post(API_ENDPOINT.LOGIN, data)
        .then(function(response) {
          $localStorage.token = response.data.token;
          $localStorage.heartNews = response.data.heartNews;
          $localStorage.likedNews = response.data.likedNews;
          $localStorage.focusedJournals = response.data.focusedJournals;
          success(response);
        })
        .catch(error);
      },
      logout: function(success) {
        clearCache();
        success();
      },
      isAuthenticated: function() {
        return $localStorage.token;
      },
      // Reserved APIs
      getTokenClaims: function() {
        return getClaimsFromToken();
      },
      saveToken: function(token) {
        $localStorage.token = token;
      },
      getToken: function() {
        return $localStorage.token;
      },
      testOldToken: function(token) {
        $localStorage.token = token;
        $localStorage.heartNews = [];
        $localStorage.likedNews = [];
        $localStorage.focusedJournals = [];
      },
      tokenExpired: function() {
        var deferred = $q.defer();
        window.plugins.toast.showShortCenter(
          "会话已过期, 重新登录",
          function(a) {},
          function(b) {}
        );

        clearCache();
        deferred.resolve('expired');
        return deferred.promise;
      }
    }
  })

  .factory('LocalService', function($localStorage, $q, $http, $state, $ionicHistory, API_ENDPOINT) {
    return {
      storeCategories: function(categories) {
        var deferred = $q.defer();
        $localStorage.categories = categories;
        if ($localStorage.categories.length)
          deferred.resolve('Saved');
        else
          deferred.reject('Unsaved');
        return deferred.promise;
      },
      categories: function() {
        return $localStorage.categories || [];
      },
      categoryName: function(categoryID) {
        var categories = $localStorage.categories || [];
        for (var i = 0; i < categories.length; i++) {
          var category = categories[i];
          if (category.sid == categoryID)
            return category.sName;
        }
        return null;
      },

      isHeartNews: function(newsID) {
        var heartNews = $localStorage.heartNews || [];
        return (heartNews.indexOf('' + newsID) !== -1);
      },

      updateHeart: function(token, newsID) {
        var deferred = $q.defer();
        $http({
          method: 'POST',
          url: API_ENDPOINT.UPDATE_HEART,
          headers: {
            'Authorization': 'bearer' + token
          },
          data: {
            newsID: newsID
          }
        })
        .then(function(response) {
          $localStorage.heartNews = response.data.heartNews;
          deferred.resolve(response);
        })
        .catch(function(error) {
          deferred.reject(error);
        })
        return deferred.promise;
      },

      isLikedNews: function(newsID) {
        var likedNews = $localStorage.likedNews || [];
        return (likedNews.indexOf('' + newsID) !== -1);
      },

      updateLiked: function(token, newsID) {
        var deferred = $q.defer();
        $http({
          method: 'POST',
          url: API_ENDPOINT.UPDATE_LIKED,
          headers: {
            'Authorization': 'bearer' + token
          },
          data: {
            newsID: newsID
          }
        })
        .then(function(response) {
          $localStorage.likedNews = response.data.likedNews;
          deferred.resolve(response);
        })
        .catch(function(error) {
          deferred.reject(error);
        })
        return deferred.promise;
      },

      isFocusedJournal: function(journalID) {
        var focusedJournals = $localStorage.focusedJournals || [];
        return (focusedJournals.indexOf('' + journalID) !== -1);
      },

      updateFocus: function(token, journalID) {
        var deferred = $q.defer();
        $http({
          method: 'POST',
          url: API_ENDPOINT.UPDATE_FOCUS,
          headers: {
            'Authorization': 'bearer' + token
          },
          data: {
            journalID: journalID
          }
        })
        .then(function(response) {
          $localStorage.focusedJournals = response.data.focusedJournals;
          deferred.resolve(response);
        })
        .catch(function(error) {
          deferred.reject(error);
        })
        return deferred.promise;
      }
    }
  })

  .factory('ModalService', function($ionicModal, $rootScope) {
    var shareModal = {
      handle: null,
      scope: $rootScope.$new()
    };

    shareModal.open = function(callback) {
      var promise;
      if (this.handle) {
        promise = shareModal.handle.show();
      } else {
        var promise = $ionicModal
          .fromTemplateUrl(
            'views/modal/share/main.html',
            {
              scope: shareModal.scope,
              animation: 'slide-in-up'
            }
          )
          .then(function(handle) {
            shareModal.handle = handle;
            promise = handle.show();
          });
      }

      this.scope.confirm = function(data) {
        shareModal.handle
          .hide()
          .then(function() {
            callback(data);
          })
      }

      // Cleanup the modal when we're done with it!
      this.scope.$on('$destroy', function() {
        this.scope.handle.remove();
      });
      // Execute action on hide modal
      this.scope.$on('modal.hidden', function() {
        // Execute action
      });
      // Execute action on remove modal
      this.scope.$on('modal.removed', function() {
        // Execute action
      });
    }

    var loginModal = {
      handle: null,
      scope: $rootScope.$new()
    };

    loginModal.open = function(callback) {
      var promise;
      if (this.handle) {
        promise = loginModal.handle.show();
      } else {
        var promise = $ionicModal
          .fromTemplateUrl(
            'views/modal/login/main.html',
            {
              scope: loginModal.scope,
              animation: 'slide-in-up'
            }
          )
          .then(function(handle) {
            loginModal.handle = handle;
            promise = handle.show();
          });
      }

      this.scope.confirm = function() {
        loginModal.handle
          .hide()
          .then(function() {
            callback();
          })
      }

      this.scope.close = function() {
        loginModal.handle.hide();
      }
      // Cleanup the modal when we're done with it!
      this.scope.$on('$destroy', function() {
        loginModal.handle.remove();
      });
      // Execute action on hide modal
      this.scope.$on('modal.hidden', function() {
        // Execute action
      });
      // Execute action on remove modal
      this.scope.$on('modal.removed', function() {
        // Execute action
      });
    }

    return {
      shareModal: shareModal,
      loginModal: loginModal
    }
  })

  .factory('ApiService', function($http, $q, $timeout, AuthService, API_ENDPOINT) {
    var PAGE_DELAY_TIME = 500; //ms

    function normalGet(url, data) {
      return $http({
        method: 'GET',
        url: url,
        params: data
      });
    }

    function normalAuthGet(url, data) {
      return $http({
        method: 'GET',
        url: url,
        headers: {
          'Authorization': 'bearer' + AuthService.getToken()
        },
        params: data
      });
    }

    function normalAuthPost(url, data) {
      return $http({
        method: 'POST',
        url: url,
        headers: {
          'Authorization': 'bearer' + AuthService.getToken()
        },
        data: data
      });
    }

    function delayGet(url, data) {
      var deferred = $q.defer();
      var _time = new Date().getTime();

      $http({
        method: 'GET',
        url: url,
        params: data
      })
      .then(function(response) {
        _time -= new Date().getTime();
        _time += PAGE_DELAY_TIME;
        if (_time > 0) {
          $timeout(function() {
            deferred.resolve(response);
          }, _time);
        } else {
          deferred.resolve(response);
        }
      })
      .catch(function(error) {
        deferred.reject(error);
      });

      return deferred.promise;
    }

    function delayAuthGet(url, data) {
      var deferred = $q.defer();
      var _time = new Date().getTime();

      $http({
        method: 'GET',
        url: url,
        headers: {
          'Authorization': 'bearer' + AuthService.getToken()
        },
        params: data
      })
      .then(function(response) {
        _time -= new Date().getTime();
        _time += PAGE_DELAY_TIME;
        if (_time > 0) {
          $timeout(function() {
            deferred.resolve(response);
          }, _time);
        } else {
          deferred.resolve(response);
        }
      })
      .catch(function(error) {
        deferred.reject(error);
      });

      return deferred.promise;
    }

    return {
      loadNewsByJournal: function(data) {
        return normalGet(API_ENDPOINT.NEWS_BY_JOURNAL, data);
      },
      loadJournal: function(data) {
        return normalGet(API_ENDPOINT.JOURNAL, data);
      },
      loadNews: function(data) {
        return normalGet(API_ENDPOINT.NEWS, data);
      },
      loadAllTopNews: function(data) {
        return normalGet(API_ENDPOINT.ALL_TOP_NEWS, data);
      },
      loadCategories: function() {
        return normalGet(API_ENDPOINT.CATEGORIES);
      },
      loadJournalsByCategory: function(data) {
        return normalGet(API_ENDPOINT.JOURNALS_BY_CATEGORY, data);
      },
      loadSearchJournals: function(data) {
        return normalGet(API_ENDPOINT.SEARCH_JOURNALS, data);
      },
      loadSearchNews: function(data) {
        return normalGet(API_ENDPOINT.SEARCH_NEWS, data);
      },
      loadRecentJournals: function(data) {
        return normalGet(API_ENDPOINT.RECENT_JOURNALS, data);
      },
      loadLatestNews: function() {
        return normalGet(API_ENDPOINT.LATEST_NEWS);
      },
      loadTopNewsOfJournal: function(data) {
        return normalGet(API_ENDPOINT.TOP_NEWS_OF_JOURNAL, data);
      },

      loadFocusedJournals: function(data) {
        return normalAuthGet(API_ENDPOINT.FOCUSED_JOURNALS, data);
      },
      loadHeartNews: function(data) {
        return normalAuthGet(API_ENDPOINT.HEART_NEWS, data);
      },
      loadLikedNews: function(data) {
        return normalAuthGet(API_ENDPOINT.LIKED_NEWS, data);
      },
      loadTopNewsOfFocusedJournals: function(data) {
        return normalAuthGet(API_ENDPOINT.TOP_NEWS_OF_FOCUSED_JOURNALS, data);
      },

      updateProfileWithoutAvatar: function(data) {
        return normalAuthPost(API_ENDPOINT.UPDATE_PROFILE_WITHOUT_AVATAR, data);
      }
    }

  })
