angular
.module('gu.controller')
.controller('UserInfoCtrl', function($scope, $rootScope, $state, $timeout,  $ionicActionSheet, $cordovaCamera, $cordovaFileTransfer, AuthService, ModalService, ApiService, API_ENDPOINT) {
  console.log('我的信息页');
  $scope.data = {
    avatar: '',
    name: '',
    gender: ''
  };

  init();

  $scope.goBack = function() {
    $rootScope.$ionicGoBack();
  }

  $scope.clickCamera = function() {
    $ionicActionSheet.show({
      buttons: [
        { text: '拍照' },
        { text: '本地图片' }
      ],
      cancelText: '取消',
      buttonClicked: function(index) {
        takePicture(1 - index);
        return true;
      }
    })
  }

  function onSaveSuccess(response) {
    if (response.status) {
      $rootScope.$broadcast('PROFILE_UPDATED', {
        avatar: response.avatarPath,
        name: $scope.data.name,
        gender: $scope.data.gender
      });

      AuthService.saveToken(response.token);
      window.plugins.toast.showShortBottom(
        "保存成功~",
        function(a) {},
        function(b) {}
      );
      $timeout(function() {
        $state.go('user');
      }, 1000);
    }
  }
  $scope.save = function() {
    if (! $scope.data.avatar || $scope.data.avatar.indexOf('http://') == 0 || $scope.data.avatar.indexOf('https://') == 0) {
      ApiService
        .updateProfileWithoutAvatar({
          name: $scope.data.name,
          gender: $scope.data.gender
        })
        .then(function(result) {
          onSaveSuccess(result.data);
        })
        .catch(function(error) {
        })
    } else {
      var options = {
        headers: {
          Authorization: 'bearer' + AuthService.getToken()
        },
        fileKey: 'avatar',
        params: {
          name: $scope.data.name,
          gender: $scope.data.gender
        }
      };

      $cordovaFileTransfer
        .upload(API_ENDPOINT.UPDATE_PROFILE, $scope.data.avatar, options)
        .then(function(result) {
          var response = JSON.parse(result.response);
          onSaveSuccess(response);
        }, function(error) {
          //alert(JSON.stringify(error));
          if (error.http_status == 401) { // 已过期， 从新登陆
            AuthService
              .tokenExpired()
              .then(function() {
                ModalService.loginModal.open($rootScope.login);
              })
          }
        }, function(progress) {
        });
    }
  }

  function init() {
    var user = AuthService.getTokenClaims();
    if (user.avatar.search("http") == 0) {
      $scope.data.avatar = user.avatar;
    } else if (user.avatar) {
      $scope.data.avatar = API_ENDPOINT.PUBLIC_FILE_URL + user.avatar;
    } else {
      $scope.data.avatar = 'img/3.png';
    }

    $scope.data.name = user.name;
    $scope.data.gender = '' + user.gender;
  }

  function takePicture(sourceType) {
    var options = {
      allowEdit: true,
      destinationType: Camera.DestinationType.FILE_URI,
      sourceType: sourceType
    };

    $cordovaCamera.getPicture(options).then(function(imageURI) {
      //alert(imageURI);
      //$scope.data.avatar = '';
      $scope.data.avatar = imageURI;
      //alert($scope.data.avatar);
    });
  }
})
