angular
.module('gu.controller', [])
.controller('RootCtrl', function($scope, $rootScope, $state, $timeout, $ionicHistory, $ionicLoading, $timeout, AuthService, LocalService, ModalService, API_ENDPOINT) {
  $rootScope.tabSelected = 'home';

  $rootScope.changeFormat = function(date) {
    var now = new Date().getTime();
    var diff = now - new Date(date).getTime(); // milliseconds
    if (diff < 86400 * 1000) {
      var day = parseInt(diff / 1000 / 3600);
      return day;
    } else {
      return date;
    }
  }

  $rootScope.goBack = function() {
    $rootScope.$ionicGoBack();
  }

  $rootScope.getCategoryName = function(categoryID) {
    return LocalService.categoryName(categoryID);
  }

  $rootScope.isHeartNews = function(newsID) {
    return LocalService.isHeartNews(newsID);
  }

  $rootScope.clickHeart = function(newsID) {
    if (! AuthService.isAuthenticated()) {
      ModalService.loginModal.open($rootScope.login);
      return;
    }

    LocalService
      .updateHeart(AuthService.getToken(), newsID)
      .then(function(response) {
        if (response.data.status) {
          $rootScope.$broadcast('HEART_CLICKED', {
            addFlag: response.data.addFlag,
            newsID: newsID
          });
          if (response.data.addFlag == true) {
            window.plugins.toast.showShortCenter(
              "收藏成功~",
              function(a) {},
              function(b) {}
            );
          }
        }
      })
      .catch(function(error) {
        if (error.status == 401) { // 已过期， 从新登陆
          AuthService
            .tokenExpired()
            .then(function() {
              ModalService.loginModal.open($rootScope.login);
            })
        }
      })
  }

  $rootScope.isLikedNews = function(newsID) {
    return LocalService.isLikedNews(newsID);
  }

  $rootScope.clickLiked = function(newsID) {
    if (! AuthService.isAuthenticated()) {
      ModalService.loginModal.open($rootScope.login);
      return;
    }

    LocalService
      .updateLiked(AuthService.getToken(), newsID)
      .then(function(response) {
        if (response.data.status) {
          $rootScope.$broadcast('LIKE_CLICKED', {
            addFlag: response.data.addFlag,
            newsID: newsID
          });

          if (response.data.addFlag == true) {
            window.plugins.toast.showShortCenter(
              "赞一个~",
              function(a) {},
              function(b) {}
            );
          }
        }
      })
      .catch(function(error) {
        if (error.status == 401) { // 已过期， 从新登陆
          AuthService
            .tokenExpired()
            .then(function() {
              ModalService.loginModal.open($rootScope.login);
            })
        }
      })
  }

  $rootScope.clickShare = function(news) {
    $rootScope.shareNews = news;
    ModalService.shareModal.open(share);
  }

  $rootScope.isFocusedJournal = function(journalID) {
    return LocalService.isFocusedJournal(journalID);
  }

  $rootScope.clickFocus = function(journalID) {
    if (! AuthService.isAuthenticated()) {
      ModalService.loginModal.open($rootScope.login);
      return;
    }

    LocalService
      .updateFocus(AuthService.getToken(), journalID)
      .then(function(response) {
        $rootScope.$broadcast('FOCUS_CLICKED', {
          addFlag: response.data.addFlag,
          journalID: journalID
        });

        if (response.data.addFlag == true) {
          window.plugins.toast.showShortCenter(
            "关注成功~",
            function(a) {},
            function(b) {}
          );
        }
      })
      .catch(function(error) {
        if (error.status == 401) { // 已过期， 从新登陆
          AuthService
            .tokenExpired()
            .then(function() {
              ModalService.loginModal.open($rootScope.login);
            })
        }
      })
  }

  function share(index) {
    var news = $rootScope.shareNews;
    var scene;
    if (index == 1) { // 朋友
       scene = Wechat.Scene.SESSION
    } else { // 朋友圈
       scene = Wechat.Scene.TIMELINE
    }
    window.Wechat && Wechat.share({
      message: {
        title: news.NTitle,
        description: news.Discription,
        thumb: news.CoverImg,
        media: {
          type: Wechat.Type.WEBPAGE,
          webpageUrl: API_ENDPOINT.H5_URL + '#/detail_news/' + news.Nid
        }
      },
      scene: scene
    }, function() {
      $timeout(function() {
        window.plugins.toast.showShortCenter(
          "分享成功~",
          function(a) {},
          function(b) {}
        );
      }, 1500);
    }, function(error) {
    });
  }

    $rootScope.testLogin = function () {
        AuthService.testLogin({}, function(response) {
            $ionicHistory.clearCache();
            $rootScope.$broadcast('LOGIN');
        }, function(error) {
        });
    };

  $rootScope.login = function() {
    if (! AuthService.isAuthenticated())
    {
        if (window.Wechat) {
            Wechat.isInstalled(function (installed) {
                if (installed) {
                    var scope = 'snsapi_userinfo';
                    var state = '_';
                    Wechat.auth(scope, state, function(response) {
                        var data = {
                            token: response.code
                        };
                        AuthService.login(data, function(response) {
                            $ionicHistory.clearCache();
                            $rootScope.$broadcast('LOGIN');
                        }, function(error) {
                        });
                    }, function(reason) {
                    })
                } else {
                    $rootScope.testLogin();
                }
            }, function () {
                $rootScope.testLogin();
            });
        } else {
            $rootScope.testLogin();
        }
    }
  }

})
